import 'dart:convert';

import 'package:flutter/foundation.dart';
import 'package:http/http.dart' as http;
import 'package:wowit/common.dart';
import 'package:wowit/methods.dart';
import 'package:shared_preferences/shared_preferences.dart';

class SQLService {
   
  static Future<Map<String, List<Map<String, Object>>>> execute(String query,
      {Map<String, Object> params , bool printLog = false}) async {
    params = setupDefaultParameter(params);
    if(printLog) print("execute query: $query, jParam ${params.toString()}");

    Codec<String, String> stringToBase64 = utf8.fuse(base64);
    String jparamsStringToBase64 = jsonEncode(params);
    if (Common.encryptAPI) {
      jparamsStringToBase64 = stringToBase64.encode(jparamsStringToBase64);
      query = stringToBase64.encode(query);
    }
    var queryParameters = {
        Common.encryptAPI ? 'a' : 'query': query,
        Common.encryptAPI ? 'aa' : 'jParam': jparamsStringToBase64,
    };
    if(printLog) print("[jsonEncode] execute query: $query, jParam  $jparamsStringToBase64");
    Map<
        String, // table namme
        List< // list row -> data tabl
            Map<String, Object> //data row
            >> retVal = {};
    String uidBackup = "";
    try {
      var response =
          await http.post(Methods.getURI(Common.encryptAPI ? '/data/exec' : '/data/exec2' ), body: queryParameters);
      if (response.statusCode == 200) {
        if (response.body == "") {
          if(printLog) print("[EMPTY RESPONSE]: $query queryParameters $queryParameters response.body is empty");
          return retVal;
        }
        var _body = response.body;
        if (Common.encryptAPI) {
          _body = stringToBase64.decode(_body);
        }
        Map<String, dynamic> jsonData =
            json.decode(_body); //--> return data set

        jsonData.forEach((key, value) {
          List<dynamic> table = value;
          List<Map<String, Object>> rsTable = [];
          for (int i = 0; i < table.length; i++) {
            rsTable.add(table[i]);
          }
          retVal[key] = rsTable;

          if (key == "uidBackup" &&
              rsTable.length > 0 &&
              rsTable[0].containsKey("uidBackup")) {
            uidBackup = rsTable[0]["uidBackup"];
          }
        });
        if(printLog) print(
            "execute query: $query, result length: ${retVal.length.toString()}");
      } else {
        if(printLog) print(
            'SQLService.execute with $query response status code = ${response.statusCode.toString()} ');
      }
    } catch (e) {
      if(printLog) print('SQLService.execute with exception:  ' + e.toString());
    }
    if (uidBackup != null &&
        uidBackup.isNotEmpty &&
        uidBackup != Common.uidBackup) {
      Common.uidBackup = uidBackup;
      var prefs = await SharedPreferences.getInstance();
      prefs.setString('uidBackup', Common.uidBackup);
    }
    
    if (printLog &&  retVal.length > 0 && retVal.containsKey('Exception')) {
      var dataException = Methods.getList(retVal, 'Exception');
      if (dataException.length > 0 && dataException[0].containsKey('Message')) {
        var exception = Methods.getString(dataException[0], 'Message');
        if (exception != "") {
          print(
              "==================>[Exception]<==================\n==================> Query:" +
                  query +
                  "\n==================> Parameter:" +
                  params.toString() +
                  "\n==================> Message:" +
                  exception.toString());
        }
      }
    }

    return retVal;
  }

  static Map<String, Object> setupDefaultParameter(Map<String, Object> params) {
    if (params == null) {
      params = {};
    }
    if (!params.containsKey('uid') && !params.containsKey('@uid')) {
      params['uid'] = Common.uid;
    }

    if (kDebugMode) {
      if (!params.containsKey('kDebugMode') &&
          !params.containsKey('@kDebugMode')) {
        params['kDebugMode'] = kDebugMode;
      }
    }

    if (!params.containsKey('languageId') &&
        !params.containsKey('@languageId') &&
        Common.languageId != null) {
      params['languageId'] = Common.languageId;
    }

    if (!params.containsKey('deviceName') &&
        !params.containsKey('@deviceName') &&
        Common.deviceName != null) {
      params['deviceName'] = Common.deviceName;
    }
    if (!params.containsKey('@deviceId') &&
        !params.containsKey('deviceId') &&
        Common.deviceId != null) {
      params['deviceId'] = Common.deviceId;
    }

    if (!params.containsKey('@mode') && !params.containsKey('mode')) {
      params['mode'] = 'App';
    }
    if (!params.containsKey('@uidPhone') &&
        !params.containsKey('uidPhone') &&
        Common.uidPhone != null) {
      params['uidPhone'] = Common.uidPhone;
    }
    if (!params.containsKey('@uidBackup') &&
        !params.containsKey('uidBackup') &&
        Common.uidBackup != null) {
      params['uidBackup'] = Common.uidBackup;
    }
    if (!params.containsKey('@fVersion') &&
        !params.containsKey('fVersion') &&
        Common.fVersion > 0) {
      params['fVersion'] = Common.fVersion;
    }

    return params;
  }

  static void logView(
      {String query = 'spLogView',
      @required int viewID,
      @required int dataID}) {
    SQLService.execute(query,
        params: {'@userID': Common.uid, '@idView': viewID, '@idData': dataID});
  }

  static void logLike(
      {String query = 'spLogLike',
      @required int idLoai,
      @required int idItem,
      @required bool isLike}) {
    SQLService.execute(query, params: {
      '@userID': Common.uid,
      '@idView': idLoai,
      '@idData': idItem,
      '@isLike': isLike
    });
  }

  static void logFollow(
      {String query = 'spLogFollow',
      @required int idLoai,
      @required int idItem,
      @required bool isFollow}) {
    SQLService.execute(query, params: {
      '@userID': Common.uid,
      '@idView': idLoai,
      '@idData': idItem,
      '@isFollow': isFollow
    });
  }
}
